import { defGetter } from "./getter.js";
import { defSetter } from "./setter.js";
/**
 * Similar to {@link defSetterUnsafe}, returns a function to update
 * values at given `path` using provided update `fn`. Paths and the
 * arguments given to the returned function are NOT type checked.
 *
 * @remarks
 * The returned function accepts a single object / array and applies
 * `fn` to given path value (incl. any additional / optional arguments
 * passed) and uses result as new value. Does not modify original state.
 *
 * The type parameter `T` can be used to indicate the type of the nested
 * value to be updated (default: `any`).
 *
 * @example
 * ```ts
 * const incB = defUpdaterUnsafe("a.b", (x, n) => x + n);
 * // or
 * const incB = defUpdaterUnsafe(["a", "b"], (x, n) => x + n);
 *
 * incB({ a: { b: 10 } }, 13);
 * // { a: { b: 23 } }
 * ```
 *
 * @param path -
 * @param fn -
 */
export const defUpdaterUnsafe = (path, fn) => defUpdater(path, fn);
export function defUpdater(path, fn) {
    const g = defGetter(path);
    const s = defSetter(path);
    return (state, ...args) => s(state, fn.apply(null, (args.unshift(g(state)), args)));
}
